==> MediaWikiLanguageExtensionBundle-2019.10.tar.bz2: extensions/Translate/src/MessageValidator/Validators/NewlineValidator.php <==
<?php

/**
 * @file
 * @author Abijeet Patro
 * @license GPL-2.0-or-later
 */

namespace MediaWiki\Extensions\Translate\MessageValidator\Validators;

use MediaWiki\Extensions\Translate\MessageValidator\Validator;
use TMessage;

/**
 * Ensures that the translation has the same number of newlines as the source
 * message at the beginning of the string.
 * @since 2019.09
 */
class NewlineValidator implements Validator {
	public function validate( TMessage $message, $code, array &$notices ) {
		$key = $message->key();
		$translation = $message->translation();
		$definition = $message->definition();

		$definitionStartNewline = $this->getStartingNewLinesCount( $definition );
		$translationStartNewline = $this->getStartingNewLinesCount( $translation );

		$failingChecks = $this->validateStartingNewline(
			$definitionStartNewline, $translationStartNewline
		);

		$this->addNotices( $failingChecks, $notices, $key, $code );
	}

	protected function getStartingNewLinesCount( $str ) {
		return strspn( $str, "\n" );
	}

	protected function getEndingNewLineCount( $str ) {
		return strspn( strrev( $str ), "\n" );
	}

	protected function validateStartingNewline(
		$definitionStartNewline, $translationStartNewline
	) {
		$failingChecks = [];
		if ( $definitionStartNewline < $translationStartNewline ) {
			// Extra whitespace at beginning
			$failingChecks[] = [
				'extra-start',
				$translationStartNewline - $definitionStartNewline
			];
		} elseif ( $definitionStartNewline > $translationStartNewline ) {
			// Missing whitespace at beginnning
			$failingChecks[] = [
				'missing-start',
				$definitionStartNewline - $translationStartNewline
			];
		}

		return $failingChecks;
	}

	protected function validateEndingNewline( $definitionEndNewline, $translationEndNewline ) {
		$failingChecks = [];
		if ( $definitionEndNewline < $translationEndNewline ) {
			// Extra whitespace at end
			$failingChecks[] = [
				'extra-end',
				$translationEndNewline - $definitionEndNewline
			];
		} elseif ( $definitionEndNewline > $translationEndNewline ) {
			// Missing whitespace at end
			$failingChecks[] = [
				'missing-end',
				$definitionEndNewline - $translationEndNewline
			];
		}

		return $failingChecks;
	}

	protected function addNotices( $failingChecks, &$notices, $key, $code ) {
		foreach ( $failingChecks as $subcheck ) {
			$notices[$key][] = [
				[ 'newline', $subcheck[0], $key, $code ],
				'translate-checks-newline-' . $subcheck[0],
				[ 'COUNT', $subcheck[1] ]
			];
		}
	}
}

==> MediaWikiLanguageExtensionBundle-2020.07.tar.bz2: extensions/Translate/src/MessageValidator/Validators/NewlineValidator.php <==
<?php
declare( strict_types = 1 );

namespace MediaWiki\Extensions\Translate\MessageValidator\Validators;

use MediaWiki\Extensions\Translate\Validation\MessageValidator;
use MediaWiki\Extensions\Translate\Validation\ValidationIssue;
use MediaWiki\Extensions\Translate\Validation\ValidationIssues;
use TMessage;

/**
 * Ensures that the translation has the same number of newlines as the source
 * message at the beginning of the string.
 * @author Abijeet Patro
 * @license GPL-2.0-or-later
 * @since 2019.09
 */
class NewlineValidator implements MessageValidator {
	public function getIssues( TMessage $message, string $targetLanguage ): ValidationIssues {
		$translation = $message->translation();
		$definition = $message->definition();

		$definitionStartNewline = $this->getStartingNewLinesCount( $definition );
		$translationStartNewline = $this->getStartingNewLinesCount( $translation );

		$failingChecks = $this->validateStartingNewline(
			$definitionStartNewline, $translationStartNewline
		);

		return $this->createIssues( $failingChecks );
	}

	protected function getStartingNewLinesCount( string $str ): int {
		return strspn( $str, "\n" );
	}

	protected function getEndingNewLineCount( string $str ): int {
		return strspn( strrev( $str ), "\n" );
	}

	protected function validateStartingNewline(
		int $definitionStartNewline,
		int $translationStartNewline
	): array {
		$failingChecks = [];
		if ( $definitionStartNewline < $translationStartNewline ) {
			// Extra whitespace at beginning
			$failingChecks[] = [
				'extra-start',
				$translationStartNewline - $definitionStartNewline
			];
		} elseif ( $definitionStartNewline > $translationStartNewline ) {
			// Missing whitespace at beginnning
			$failingChecks[] = [
				'missing-start',
				$definitionStartNewline - $translationStartNewline
			];
		}

		return $failingChecks;
	}

	protected function validateEndingNewline(
		int $definitionEndNewline,
		int $translationEndNewline
	): array {
		$failingChecks = [];
		if ( $definitionEndNewline < $translationEndNewline ) {
			// Extra whitespace at end
			$failingChecks[] = [
				'extra-end',
				$translationEndNewline - $definitionEndNewline
			];
		} elseif ( $definitionEndNewline > $translationEndNewline ) {
			// Missing whitespace at end
			$failingChecks[] = [
				'missing-end',
				$definitionEndNewline - $translationEndNewline
			];
		}

		return $failingChecks;
	}

	protected function createIssues( array $failingChecks ): ValidationIssues {
		$issues = new ValidationIssues();
		foreach ( $failingChecks as [ $subType, $count ] ) {
			$issue = new ValidationIssue(
				'newline',
				$subType,
				"translate-checks-newline-$subType",
				[ 'COUNT', $count ]
			);

			$issues->add( $issue );
		}

		return $issues;
	}
}
